/* ============================================================
 * File  : cdarchiving.h
 * Author: Gilles Caulier <caulier.gilles@free.fr>
 * Date  : 2003-09-05
 * Description : Digikam Albums Cd Archiving
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef CDARCHIVING_H
#define CDARCHIVING_H

#include <unistd.h>

#include <qobject.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qfont.h>
#include <qdatetime.h>
#include <qimage.h>
#include <qprogressdialog.h>
#include <qtextcodec.h>
#include <qstringlist.h>

#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>
#include <kio/global.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kaction.h>
#include <kglobal.h>
#include <klocale.h>
#include <kcharsets.h>
#include <kmessagebox.h>
#include <kurl.h>
#include <kapplication.h>
#include <kimageio.h>
#include <kdebug.h>
#include <kgenericfactory.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kprocess.h>

#include <digikam/albummanager.h>
#include <digikam/albuminfo.h>
#include "cdarchivingdialog.h"
#include "plugin_cdarchiving.h"

#define NAV_THUMB_MAX_SIZE 64


class QProgressDialog;
class KURL;
class KIGPDialog;

typedef QMap<QString,QString> CommentMap;

class CDArchiving : public QObject
{
Q_OBJECT

public:
  CDArchiving(QObject *parent=0, KAction *action_cdarchiving=0);
  virtual ~CDArchiving();

  void writeSettings(void);
  void readSettings(void);

public slots:
  void Activate();
  void slotCancelled();
  void K3bDone(KProcess*);
  
private:
  KConfig            *config;
  CDArchivingDialog  *m_configDlg;
  Digikam::AlbumInfo *album;
  KAction            *actionCDArchiving;
  KProcess           *Proc;

  bool               m_cancelled;
  bool               m_recurseSubDirectories;
  bool               m_copyFiles;
  bool               m_useCommentFile;

  QString            AlbumTitle;
  QString            AlbumComments;
  QString            AlbumCollection;
  QString            AlbumDate;
  QString            m_StreamMainPageAlbumPreview;
  QString            m_imagesFileFilter;

  QString            HTMLInterfaceFolder;
  QString            HTMLInterfaceIndex;
  QString            HTMLInterfaceAutoRunInf;
  QString            HTMLInterfaceAutoRunFolder;
  QString            m_tmpFolder;

  int                m_imgWidth;
  int                m_imgHeight;
  int                m_imagesPerRow;
  int                m_LevelRecursion;

  int                m_targetImgWidth;
  int                m_targetImgHeight;

  QProgressDialog    *m_progressDlg;
  CommentMap         *m_commentMap;

  bool buildHTMLInterface (void);

  bool createDirectory(QDir thumb_dir, QString imgGalleryDir, QString dirName);

  void createHead(QTextStream& stream);
  void createCSSSection(QTextStream& stream);
  void createBody(QTextStream& stream, const QString& sourceDirName, const QStringList& subDirList, const QDir& imageDir, const KURL& url, const QString& imageFormat);

  bool createThumb( const QString& imgName, const QString& sourceDirName, const QString& imgGalleryDir, const QString& imageFormat);
  bool ResizeImage( const QString Path, const QString Directory, const QString ImageFormat, const QString ImageNameFormat, int *Width, int *Height, int SizeFactor, bool ColorDepthChange, int ColorDepthValue, bool CompressionSet, int ImageCompression);
  bool createHtml( const KURL& url, const QString& sourceDirName, int recursionLevel, const QString& imageFormat);
  bool createPage(const QString& imgGalleryDir , const QString& imgName, const QString& previousImgName, 
		  const QString& nextImgName, const QString& comment, const QString& imageFormat, const QString& AlbumTitle, const QString& sourceDirName);

  void createBodyMainPage(QTextStream& stream, KURL& url);
  void loadComments(void);
  static QString extension(const QString& imageFormat);
  bool BuildK3bXMLprojectfile (QString HTMLinterfaceFolder, QString IndexHtm, QString AutoRunInf, QString AutorunFolder);
  bool AddFolderTreeToK3bXMLProjectFile (QString dirname, QTextStream* stream);
  QString formatString2HTML( const QString& string );
  bool CreateAutoRunInfFile(void);
  bool DeleteDir(QString dirname);
  bool deldir(QString dirname);
};

#endif /* CDARCHIVING_H */
